/*******************************************************************************
*                                 ZM4xx Demo
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Stock Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
* e-mail:      ametal.support@zlg.cn
*******************************************************************************/

/**
 * \file
 * \brief ZM4xx Demo
 *
 * \internal
 * \par Modification history
 * - 1.00 17-11-22  sup, first implementation.
 * \endinternal
 */
#include "LPC11xx.h"                                                    /* LPC11xxĴ            */
#include "uart.h"
#include "stdint.h"
#include "radio.h"
#include "int.h"
#include "timer.h"

#define LED_INIT()    LPC_GPIO2->DIR  |=  0XFF0                         /* LEDʼ                    */
#define FREQ          (433000000)
#define TEST_TIME     60 //second

extern uint16_t   uiByteNum;                                            /*  ڽݸ            */

uint8_t     g_data_buf[PACK_LENGTH] = {0};
uint16_t    g_tx_enable             = 0;
uint16_t    g_time_out              = 0;
uint16_t    g_mode = 0;

static uint8_t        __g_ucsleep  = 0;
static uint8_t        __g_led      = 0;
static uint8_t        __g_pa_level = 0;                                /*  书,С5dBm,20dBm */
static uint16_t       __g_rcv_len  = 0;                                /*  ߽ճ  */
static radio_handle_t __gp_handle;
static uint16_t       __g_state;
static uint32_t       __g_pkts_tmp = 0;
static uint32_t       __g_pkt_rx_cnt = 0;

static void __delay_ms (uint32_t ms);
static void __led_show (uint8_t ucled);

/**
 * \brief  ģжϷ
 */
void radio_recv_int_handle (void)
{
    int      ret = 0;
    uint8_t  len;
    
    if (!__g_pkt_rx_cnt && (radio_state_get(__gp_handle) == RX_RUNING_ST)) {
        timer0_32_start();
    }
    ret = radio_buf_recv(__gp_handle, g_data_buf, &len);  /*  ն  */
    if (ret == 0) {
        __g_rcv_len += len;
    }
}




/**
 * \brief  S2ӦжϷһ
 */
void key2_int_handle (void)
{
    uint16_t  i = 0;

    __delay_ms(10);
    while(!(LPC_GPIO1->DATA & (1ul << 2)));                                /*  ȴɿ                */
    LPC_GPIO1->IC |= (1ul << 2);                                        /*  жϱ־                */

    for (i = 0; i < 100; i++) {
        g_data_buf[i] = i;
    }

    uiByteNum  = 63;
    g_tx_enable = 1;                                                      /* ʹܷ                 */

    LPC_GPIO1->IC |= (1ul << 2);                                        /*  жϱ־                */
}




/**
 * \brief  S3жϷ,ı
 */
void key3_int_handle (void) 
{
    uint8_t i = 0;
    
    /* ÿΰһС   */
    __g_pa_level++;
    if (__g_pa_level > 7) {
        __g_pa_level = 0;
    }

    while (!(LPC_GPIO0->DATA & (1ul << 7)));
    /*  ȴɿ                */
    __delay_ms(10);
    LPC_GPIO0->IC |= (1ul << 7);                                        /*  жϱ־               */
    radio_pa_set(__gp_handle, __g_pa_level);
    __led_show(__g_pa_level);
}




static void __led_show (uint8_t ucled)
{
    uint32_t temp = 0;

     /* ȵIO˳,ٸֵ */
    temp |= (ucled & (1ul << 7)) >> 5;   /* LED7,DIO6 */
    temp |= (ucled & (1ul << 6)) >> 3;   /* LED6,DIO7 */
    temp |= (ucled & (1ul << 5)) >> 1;   /* LED5,DIO8 */
    temp |= (ucled & (1ul << 4)) >> 4;   /* LED4,DIO4 */
    temp |= (ucled & (1ul << 3)) >> 2;   /* LED3,DIO5 */
    temp |= (ucled & (1ul << 2)) << 3;   /* LED2,DIO9 */
    temp |= (ucled & (1ul << 1)) << 5;   /* LED1,DIO10 */
    temp |= (ucled & (1ul << 0)) << 7;   /* LED0,DIO11 */

    LPC_GPIO2->DATA &= 0x00f;
    LPC_GPIO2->DATA |= ~(temp << 4);
}




static void __delay_ms (uint32_t ms)
{
    uint32_t i = 0;
    uint32_t j = 0;

    while (i < ms) {
        i++;
        for(j = 0; j < 4369; j++);
    }
}



/**
 * \brief  S2Ӧжϳʼ
 */
static void __key2_int (void)
{
    LPC_GPIO1->DIR &= ~ (1ul << 2);                                     /*  P1.2Ϊ              */
    LPC_GPIO1->IS   = 0x00;                                             /*  P1.2Ϊж              */
    LPC_GPIO1->IEV &= (1ul << 2);                                       /*  ½ж                  */
    LPC_GPIO1->IE  |= (1ul << 2);                                       /*  P1.2жϲ              */
    NVIC_EnableIRQ(EINT1_IRQn);                                         /* жϲʹ               */
    NVIC_SetPriority(EINT1_IRQn, 2);
}



/**
 * \brief  S3жϳʼ
 */
static void __key3_int (void)
{
    LPC_GPIO0->DIR &= ~ (1ul << 7);                                     /*  P0.7Ϊ              */
    LPC_GPIO0->IS   = 0x00;                                             /*  P0.7Ϊж              */
    LPC_GPIO0->IEV &= ~(1ul << 7);                                      /*  ½ж                  */
    LPC_GPIO0->IE  |= (1ul << 7);                                       /*  P0.7жϲ              */
    NVIC_EnableIRQ(EINT0_IRQn);                                         /* жϲʹ               */
    NVIC_SetPriority(EINT0_IRQn, 2);
    LPC_GPIO0->IC |= (1ul << 7);                                        /*  жϱ־                */
}




/**
 * \brief  رհж
 */
static void __disable_key_irq (void)
{
    NVIC_DisableIRQ(EINT0_IRQn);                  /* ʧжϣΪжжģSPIʱ  */
    NVIC_DisableIRQ(EINT1_IRQn);
}



/**
 * \brief  ָж
 */
static void __enable_key_irq (void)
{
    NVIC_EnableIRQ(EINT0_IRQn);                                         /* ָж                     */
    NVIC_EnableIRQ(EINT1_IRQn);
}



void demo_zm4xx_data_rate_test_entry (radio_handle_t handle)
{
    uint16_t i = 0;
    double   second = 0;
    double   timeout = 0;
    uint8_t  pkt_len = 0;
    uint32_t tmp;
    uint32_t pkt_tx_cnt = 0, pkt_correct = 0, pkt_error = 0; 
    
    __gp_handle = handle;

    /* ʼLED */
    LED_INIT();

    /* ʼUART  */
    uartInit();

    /* ʼʱ */
    timer0_32_init();

    /* ʼťS3ж;Ӧ */
    __key3_int();

    /* ʼťS2ж */
    __key2_int();
    
    radio_frq_set(handle, FREQ);

    /*  Ϊ״̬  */
    radio_mode_set(handle, RX_MODE);

    /*  ʼģж  */
    radio_int_init();

    __led_show(0x18);

    __g_rcv_len = 0;
    
    while (1) {
        /*    */
        if (__g_rcv_len > 0) {
            /* ¼ */
            pkt_len = __g_rcv_len;
            
            /*  У */
            for (i = 0; i < __g_rcv_len; i++) {
                if (g_data_buf[i] != i) {
                    break;
                }
            }
            
            __g_pkt_rx_cnt++;
            if (i >= __g_rcv_len) {
                pkt_correct++;
            } else {
                pkt_error++;
            }
            
            second = (double)timer0_32_tc_get() / 1000000;

            __g_rcv_len = 0;
            radio_mode_set(handle, RX_MODE);
        }

        /*    */
        if (g_tx_enable == 1) {              /*  ճʱ  */
            if (pkt_tx_cnt == 0) {
                timer0_32_start();
            }
       
            __led_show(0x80);

            radio_buf_send(handle, g_data_buf, uiByteNum);       /*  Ͷ  */
            pkt_tx_cnt++;
            
            second = (double)timer0_32_tc_get() / 1000000;
            if (second >= TEST_TIME) {
                printf("tx data rate:%fbps time:%fs\r\n", (double)pkt_tx_cnt * uiByteNum * 8 / second, second);
                printf("tx pkt_cnt:%d\r\n", pkt_tx_cnt);
                pkt_tx_cnt = 0;
                g_tx_enable = 0;
                while(1);
            }
            
            __delay_ms(1);
        }
        
        /* ճʱ */
        timeout = (double)timer0_32_tc_get() / 1000000;
        if ((timeout - second) > 3) {
            printf("rx data rate:%fbps time:%fs\r\n", (double)__g_pkt_rx_cnt * pkt_len * 8 / second, second);
            printf("pkt_cnt:%d pkt_correct:%d pkt_error:%d\r\n", __g_pkt_rx_cnt, pkt_correct, pkt_error);
            __g_pkts_tmp = 0;
            while(1);
        }
    }
}
