/*******************************************************************************
*                                 Radio interface
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2017 Guangzhou ZHIYUAN Electronics Stock Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
* e-mail:      ametal.support@zlg.cn
*******************************************************************************/

/**
 * \file
 * \brief ģͨýӿ
 *
 * \internal
 * \par Modification history
 * - 1.00 17-11-22  sup, first implementation.
 * \endinternal
 */

#include "radio.h"
#include "string.h"




int radio_reset (radio_handle_t handle)
{
    if (handle == NULL) {
        return RADIO_RET_PARAM_INVALID;
    }
    
    return handle->p_funcs->pfn_radio_reset(handle->p_drv);
}



int radio_sync_id_set (radio_handle_t handle, uint8_t *p_id, uint8_t len)
{
    if (handle == NULL || p_id == NULL) {
        return RADIO_RET_PARAM_INVALID;
    }
    
    return handle->p_funcs->pfn_radio_id_set(handle->p_drv, p_id, len);
}



int radio_sync_id_get (radio_handle_t handle, uint8_t *p_id, uint8_t *p_len)
{
    if (handle == NULL || p_id == NULL) {
        return RADIO_RET_PARAM_INVALID;
    }
    
    return handle->p_funcs->pfn_radio_id_get(handle->p_drv, p_id, p_len);
}



int radio_preamble_length_set (radio_handle_t handle, uint16_t size)
{
    if (handle == NULL) {
        return RADIO_RET_PARAM_INVALID;
    }
    
    return handle->p_funcs->pfn_radio_preamble_length_set(handle->p_drv, size);
}



int radio_preamble_length_get (radio_handle_t handle, uint16_t *p_len)
{   
    return handle->p_funcs->pfn_radio_preamble_length_get(handle->p_drv, p_len);
}



int radio_pa_set (radio_handle_t handle, uint8_t level, float *p_power)
{
    if ( handle == NULL ) {
        return RADIO_RET_PARAM_INVALID;
    }

    return handle->p_funcs->pfn_radio_pa_set(handle->p_drv, level, p_power);
}



int radio_freq_set (radio_handle_t handle, uint32_t freq)
{
    if (handle == NULL) {
        return RADIO_RET_PARAM_INVALID;
    }

    return handle->p_funcs->pfn_radio_freq_set(handle->p_drv, freq);
}



int radio_rssi_read (radio_handle_t handle, int16_t *p_rssi)
{
    if ((handle == NULL) || (p_rssi == NULL)) {
        return RADIO_RET_PARAM_INVALID;
    }
    
    return handle->p_funcs->pfn_radio_rssi_read(handle->p_drv, p_rssi);
}



int radio_mode_set (radio_handle_t handle, uint8_t mode)
{
    if (handle == NULL) {
        return RADIO_RET_PARAM_INVALID;
    }

    return handle->p_funcs->pfn_radio_mode_set(handle->p_drv, mode);
}



int radio_state_get (radio_handle_t handle, radio_state_t *p_state)
{
    if ((handle == NULL) || (p_state == NULL)) {
        return RADIO_RET_PARAM_INVALID;
    }
    
    return handle->p_funcs->pfn_radio_state_get(handle->p_drv, p_state);
}



int radio_buf_send (radio_handle_t handle, uint8_t *buf, uint8_t size)
{
    if ((handle == NULL) || (buf == NULL)) {
        return RADIO_RET_PARAM_INVALID;
    }

    return handle->p_funcs->pfn_radio_buf_send(handle->p_drv, buf, size);
}



int radio_buf_recv (radio_handle_t handle, uint8_t *buf, uint8_t *p_size)
{
    if ((handle == NULL) || (buf == NULL) || (p_size == NULL)) {
        return RADIO_RET_PARAM_INVALID;
    }

    return handle->p_funcs->pfn_radio_buf_recv(handle->p_drv, buf, p_size);
}




int radio_ioctl (radio_handle_t handle, int cmd, void *p_arg)
{
    if (handle == NULL) {
        return RADIO_RET_PARAM_INVALID;
    } 
    
    return handle->p_funcs->pfn_radio_ioctl(handle->p_drv, cmd, p_arg);
}




int radio_dio0_irq_func (radio_handle_t handle)
{
    if (handle == NULL) {
        return RADIO_RET_PARAM_INVALID;
    }
    
    if (handle->p_funcs->pfn_radio_dio0_irq_func != NULL) {
        handle->p_funcs->pfn_radio_dio0_irq_func(handle->p_drv);
		return RADIO_RET_OK;
    } else {
		return RADIO_RET_MODULE_NOT_SUPPORT;
	}
}




int radio_dio1_irq_func (radio_handle_t handle)
{
    if (handle == NULL) {
        return RADIO_RET_PARAM_INVALID;
    }
    
    if (handle->p_funcs->pfn_radio_dio1_irq_func != NULL) {
        handle->p_funcs->pfn_radio_dio1_irq_func(handle->p_drv);
		return RADIO_RET_OK;
    } else {
		return RADIO_RET_MODULE_NOT_SUPPORT;
	}
}




int radio_dio2_irq_func (radio_handle_t handle)
{
    if (handle == NULL) {
        return RADIO_RET_PARAM_INVALID;
    }
    
    if (handle->p_funcs->pfn_radio_dio2_irq_func != NULL) {
        handle->p_funcs->pfn_radio_dio2_irq_func(handle->p_drv);
		return RADIO_RET_OK;
    } else {
		return RADIO_RET_MODULE_NOT_SUPPORT;
	}
}




int radio_dio3_irq_func (radio_handle_t handle)
{
    if (handle == NULL) {
        return RADIO_RET_PARAM_INVALID;
    }
    
    if (handle->p_funcs->pfn_radio_dio3_irq_func != NULL) {
        handle->p_funcs->pfn_radio_dio3_irq_func(handle->p_drv);
		return RADIO_RET_OK;
    } else {
		return RADIO_RET_MODULE_NOT_SUPPORT;
	}
}




int radio_dio5_irq_func (radio_handle_t handle)
{
    if (handle == NULL) {
        return RADIO_RET_PARAM_INVALID;
    }
    
    if (handle->p_funcs->pfn_radio_dio5_irq_func != NULL) {
        handle->p_funcs->pfn_radio_dio5_irq_func(handle->p_drv);
		return RADIO_RET_OK;
    } else {
		return RADIO_RET_MODULE_NOT_SUPPORT;
	}
}
