/*******************************************************************************
*                                 Radio interface
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2017 Guangzhou ZHIYUAN Electronics Stock Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
* e-mail:      ametal.support@zlg.cn
*******************************************************************************/

/**
 * \file
 * \brief ģͨýӿ
 *
 * \internal
 * \par Modification history
 * - 1.00 17-11-22  sup, first implementation.
 * \endinternal
 */


#ifndef __RADIO_H_
#define __RADIO_H_

#include "stdint.h"
#include "radio_differ.h"


/**
 * \brief ģʽ
 */
#define SLEEP_MODE                       0x00
#define STANDBY_MODE                     0x01
#define TX_MODE                          0x02
#define RX_MODE                          0x03



/**
 * \brief ӿڷֵ
 */
#define RADIO_RET_OK                      0
#define RADIO_RET_ERR                    -1 
#define RADIO_RET_CMD_NOT_SUPPORT        -2
#define RADIO_RET_PARAM_INVALID          -3
#define RADIO_RET_CFG_OUT_OF_RANGE       -4
#define RADIO_RET_FREQ_NOT_SUPPORT       -5
#define RADIO_RET_MODULE_NOT_SUPPORT     -6
#define RADIO_RET_NOT_RECV_PKT           -7
#define RADIO_RET_NOT_IN_RX_MODE         -8



/**
 * \brief ģָ
 */
struct radio_drv_funcs {
    int           (*pfn_radio_reset)               (void *p_drv);
    int           (*pfn_radio_id_set)              (void *p_drv, uint8_t *p_id, uint8_t id_len);
    int           (*pfn_radio_id_get)              (void *p_drv, uint8_t *p_id, uint8_t *id_len);
    int           (*pfn_radio_preamble_length_set) (void *p_drv, uint16_t size);
    int           (*pfn_radio_preamble_length_get) (void *p_drv, uint16_t *p_len);
    int           (*pfn_radio_pa_set)              (void *p_drv, uint8_t level, float *p_power);
    int           (*pfn_radio_freq_set)            (void *p_drv, uint32_t freq);
    int           (*pfn_radio_mode_set)            (void *p_drv, uint8_t mode);
    int           (*pfn_radio_state_get)           (void *p_drv, radio_state_t *p_state);
    int           (*pfn_radio_rssi_read)           (void *p_drv, int16_t *p_rssi);
    int           (*pfn_radio_buf_send)            (void *p_drv, uint8_t *buf, uint8_t size);
    int           (*pfn_radio_buf_recv)            (void *p_drv, uint8_t *buf, uint8_t *p_size);
    int           (*pfn_radio_ioctl)               (void *p_drv, int cmd, void *p_arg);
    void          (*pfn_radio_dio0_irq_func)       (void *p_drv);
    void          (*pfn_radio_dio1_irq_func)       (void *p_drv);
    void          (*pfn_radio_dio2_irq_func)       (void *p_drv);
    void          (*pfn_radio_dio3_irq_func)       (void *p_drv);
    void          (*pfn_radio_dio5_irq_func)       (void *p_drv);
};



/**
 * \brief Ƶ豸
 */
typedef struct radio_dev {
    const struct radio_drv_funcs *p_funcs;   /* 豸  */
    void                         *p_drv;
} radio_dev_t;



typedef radio_dev_t *radio_handle_t;



/**
 * \brief ģ鸴λ
 *
 * \param[in]  handle   : ģ
 *
 * \retval  RADIO_RET_OK            : λɹ
 * \retval  RADIO_RET_PARAM_INVALID : Ч
 */
int radio_reset (radio_handle_t handle);



/**
 * \brief ģͬID
 *
 * \param[in]  handle   : ģ
 * \param[in]  p_id     : IDݵָ
 * \param[in]  id_len   : ID
 *
 * \retval  RADIO_RET_OK                 : óɹ
 * \retval  RADIO_RET_PARAM_INVALID      : Ч
 * \retval  RADIO_RET_CFG_OUT_OF_RANGE   : IDȳΧ
 * \retval  RADIO_RET_MODULE_NOT_SUPPORT : ģ鲻ָ֧ù
 */
int radio_sync_id_set (radio_handle_t handle, uint8_t *p_id, uint8_t id_len);



/**
 * \brief ȡģͬID
 *
 * \param[in]  handle   : ģ
 * \param[in]  p_id     : IDݵָ
 * \param[in]  p_len    : ID
 *
 * \retval  RADIO_RET_OK                 : ȡɹ
 * \retval  RADIO_RET_PARAM_INVALID      : Ч
 * \retval  RADIO_RET_MODULE_NOT_SUPPORT : ģ鲻ָ֧ù
 */
int radio_sync_id_get (radio_handle_t handle, uint8_t *p_id, uint8_t *p_len);



/**
 * \brief ģǰ볤
 *
 * \param[in]  handle   : ģ
 * \param[in]  size     : IDݵָ
 *
 * \retval  RADIO_RET_OK               : óɹ
 * \retval  RADIO_RET_PARAM_INVALID    : Ч
 * \retval  RADIO_RET_CFG_OUT_OF_RANGE : ǰ볤ȳΧ
 */
int radio_preamble_length_set (radio_handle_t handle, uint16_t size);



/**
 * \brief ȡģǰ볤
 *
 * \param[in]  handle   : ģ
 * \param[out] p_len    : ǰ볤
 *
 * \retval  RADIO_RET_OK            : ȡɹ
 * \retval  RADIO_RET_PARAM_INVALID : Ч
 */
int radio_preamble_length_get (radio_handle_t handle, uint16_t *p_len);



/**
 * \brief ģ鷢书ʵλ0~7
 *
 * \param[in]  handle   : ģ
 * \param[in]  level    : ʵλ
 * \param[out] p_power  : ʵʵù
 *
 * \retval  RADIO_RET_OK            : óɹ
 * \retval  RADIO_RET_ERR           : ʧ
 * \retval  RADIO_RET_PARAM_INVALID : Ч
 */
int radio_pa_set (radio_handle_t handle, uint8_t level, float *p_power);



/**
 * \brief ģƵ
 *
 * \param[in]  handle   : ģ
 * \param[in]  freq     : Ƶ
 *
 * \retval  RADIO_RET_OK               : óɹ
 * \retval  RADIO_RET_PARAM_INVALID    : Ч
 * \retval  RADIO_RET_CFG_OUT_OF_RANGE : ƵʳΧ
 * \retval  RADIO_RET_FREQ_NOT_SUPPORT : ָ֧Ƶ
 */
int radio_freq_set (radio_handle_t handle, uint32_t freq);



/**
 * \brief ģ鵱ǰRSSIֵ
 *
 * \param[in]  handle : ģ
 * \param[out] p_rssi : RSSI
 *
 * \retval  RSSIֵ
 */
int radio_rssi_read (radio_handle_t handle, int16_t *p_rssi);



/**
 * \brief ģģʽ
 *
 * \param[in]  handle   : ģ
 * \param[in]  mode     : ģʽ
 *
 * \retval  RADIO_RET_OK                 : óɹ
 * \retval  RADIO_RET_PARAM_INVALID      : Ч
 * \retval  RADIO_RET_MODULE_NOT_SUPPORT : ģ鲻ָ֧ģʽ
 */
int radio_mode_set (radio_handle_t handle, uint8_t mode);



/**
 * \brief ȡģ鵱ǰ״̬
 *
 * \param[in]  handle  : ģ
 * \param[out] p_state : ģ״̬
 *
 * \retval  RADIO_RET_OK                 : ȡɹ
 * \retval  RADIO_RET_PARAM_INVALID      : Ч
 * \retval  RADIO_RET_MODULE_NOT_SUPPORT : ģ鲻ָ֧ģʽ
 */
int radio_state_get (radio_handle_t handle, radio_state_t *p_state);



/**
 * \brief ģ鷢
 *
 * \param[in]  handle   : ģ
 * \param[in]  p_buf    : ݻ
 * \param[in]  size     : ݳ
 *
 * \retval  RADIO_RET_OK               : ͳɹ
 * \retval  RADIO_RET_PARAM_INVALID    : Ч
 * \retval  RADIO_RET_CFG_OUT_OF_RANGE : ȳΧ
 */
int radio_buf_send (radio_handle_t handle, uint8_t *p_buf, uint8_t size);



/**
 * \brief ģ
 *
 * \param[in]  handle   : ģ
 * \param[in]  p_buf    : ݻ
 * \param[in]  size     : յݵĳ
 *
 * \retval  RADIO_RET_OK            : ճɹ
 * \retval  RADIO_RET_PARAM_INVALID : Ч
 * \retval  RADIO_RET_NOT_RECV_PKT  : ûнյݰ
 */
int radio_buf_recv (radio_handle_t handle, uint8_t *p_buf, uint8_t *p_size);



/**
 * \brief ģƺ
 *
 * \param[in]  handle   : ģ
 * \param[in]  cmd      : 
 * \param[in]  p_arg    : ӦĲ
 *
 * \retval  RADIO_RET_OK              : ִгɹ
 * \retval  RADIO_RET_ERR             : ִʧ
 * \retval  RADIO_RET_PARAM_INVALID   : Ч
 * \retval  RADIO_RET_CMD_NOT_SUPPORT : ָ֧
 */
int radio_ioctl (radio_handle_t handle, int cmd, void *p_arg);



/**
 * \brief ģDIO0жϷ
 *
 * \param[in]  handle   : ģ
 *
 * \retval  RADIO_RET_OK                 : жϷִ
 * \retval  RADIO_RET_MODULE_NOT_SUPPORT : ģжϷ
 */
int radio_dio0_irq_func (radio_handle_t handle);
 
 
 
/**
 * \brief ģDIO1жϷ
 *
 * \param[in]  handle   : ģ
 *
 * \retval  RADIO_RET_OK                 : жϷִ
 * \retval  RADIO_RET_MODULE_NOT_SUPPORT : ģжϷ
 */
int radio_dio1_irq_func (radio_handle_t handle);
 
 
 
/**
 * \brief ģDIO2жϷ
 *
 * \param[in]  handle   : ģ
 *
 * \retval  RADIO_RET_OK                 : жϷִ
 * \retval  RADIO_RET_MODULE_NOT_SUPPORT : ģжϷ
 */
int radio_dio2_irq_func (radio_handle_t handle);
 

 
/**
 * \brief ģDIO3жϷ
 *
 * \param[in]  handle   : ģ
 *
 * \retval  RADIO_RET_OK                 : жϷִ
 * \retval  RADIO_RET_MODULE_NOT_SUPPORT : ģжϷ
 */
int radio_dio3_irq_func (radio_handle_t handle);
 
 
 
/**
 * \brief ģDIO5жϷ
 *
 * \param[in]  handle   : ģ
 *
 * \retval  RADIO_RET_OK                 : жϷִ
 * \retval  RADIO_RET_MODULE_NOT_SUPPORT : ģжϷ
 */
int radio_dio5_irq_func (radio_handle_t handle);
 
#endif /* _RADIO_H_ */
